let parse = require("./parse.js");

let express = require('express');
let app = express();

let localPort = 4646; //GoGo
let finalStr;
let errorObject;

function parseGoGoMetaDataArgsUrl() {
    retVal = '';

    // skip the first two arguments - root file path and main.js path arguments
    var inputArgs = process.argv.slice(2);

    if (typeof inputArgs !== 'undefined' && inputArgs.length > 0) {
        retVal = inputArgs[0];
    } else {
    }

    return retVal;
}

var gogoMetaDataUrl = parseGoGoMetaDataArgsUrl();

function isEmpty(str) {
    return (!str || 0 === str.length);
}

function isExternal(aUrl) {
    var location = {};
    var match = aUrl.match(/^([^:\/?#]+:)?(?:\/\/([^\/?#]*))?([^?#]+)?(\?[^#]*)?(#.*)?/);

    if (match != null && typeof match[1] === 'string' &&
        match[1].length > 0 && match[1].toLowerCase() !== location.protocol)
        return true;

    return match != null && typeof match[2] === 'string' &&
        match[2].length > 0 &&
        match[2].replace(new RegExp(':(' + { 'http:': 80, 'https:': 443 }[location.protocol] + ')?$'), '') !==
        location.host
}

function getHostName(aUrl) {
    var match = aUrl.match(/:\/\/(www[0-9]?\.)?(.[^/:]+)/i);

    if (match != null && match.length > 2 && typeof match[2] === 'string' && match[2].length > 0) {
        return match[2];
    } else {
        return null;
    }
}

const request = () => new Promise(function(myResolve, myReject) {
    console.log("loadUrl -> gogoMetaDataUrl: "+ gogoMetaDataUrl);
    if (!isEmpty(gogoMetaDataUrl) && isExternal(gogoMetaDataUrl)) {
        var http;

        if (gogoMetaDataUrl.includes("https")) {
            http = require('https');
        } else {
            http = require('http');
        }

        var options = {
            host: getHostName(gogoMetaDataUrl),
            path: gogoMetaDataUrl,
            method: 'GET',
            timeout: 10000,
            headers: { 'user-agent': 'node.js' }
        }
        var request = http.request(options, function(res) {
            var data = '';
            res.on('data', function(chunk) {
                console.log("loadUrl -> request -> on data -> chunk: " + chunk);
                data += chunk;
            });

            res.on('end', function() {
                console.log("loadUrl -> request -> on end");
                try {
                    finalStr = parse.transform(data.toString());
                    myResolve(finalStr);
                } catch (error) {
                    console.log("loadUrl -> request -> on end -> catch error: " + error);
                    errorObject = error;
                    myReject(errorObject);
                }
            });
        });

        request.on('error', function(e) {
            console.log("loadUrl -> request -> on error: " + e);
            errorObject = e;
            myReject(errorObject);
        });
        request.on('timeout', function() {
            errorObject = "ETIMEOUT";
            console.log('Timeout: can not reach to Metadata API!');
            myReject(errorObject);
        });
        console.log("loadUrl -> request -> end");
        request.end();
    } else {
        console.log("loadUrl -> empty url provided");
    }
});

let doRequest = request();

doRequest.then(function(value) {
        console.log("doRequest Resolved.");
    },
    function(error) {
        console.log("doRequest Rejected.");
        doRequest = undefined;
    });

app.get('/metadata', function(req, res) {
    (function respond() {
        if (!doRequest) {
            doRequest = request();
        }
        doRequest.then(
            function(value) {
                res.setHeader("Content-Type", "application/json;charset=UTF-8");
                res.end(value);
            },
            function(error) {
                doRequest = undefined;
                res.setHeader("Content-Type", "application/json;charset=UTF-8");
                res.end(parse.errorHandler(finalStr, error));
            }
        );
    })();
});

var server = app.listen(localPort);
