let MediaItem = require("./mediaitem.js")
let SubMediaItem = require("./submediaitem.js")
let MediaSubCategory = require("./mediasubcategory.js");
let MediaSuperCategory = require("./mediasupercategory.js");
let Utils = require("./utils.js");
let { urop } = require('./urop.js');

let filters = require("./filters.json");
let jsonQuery = require('json-query');

let utils = new Utils();
let finalStr;

var tvshows = { 'tvshows': [] };

function get_talents(item, type) {
    let response = '';

    if (!type) {
        return response;
    }

    if ((item || {}).hasOwnProperty('talents') && item.talents) {
        let k = 0;
        for (k = 0; k < item.talents.length; k++) {
            if (item.talents[k] && item.talents[k].type == type) {
                if (response.length > 0) {
                    response += ', ';
                }
                response += item.talents[k].id;
            }
        }
    }
    return response;
}

function get_category_array(item) {
    let response = [];

    if ((item || {}).hasOwnProperty('taxonomyRefs') && item.taxonomyRefs) {
        let k = 0;
        for (k = 0; k < item.taxonomyRefs.length; k++) {
            if (item.taxonomyRefs[k]) {
                response.push(item.taxonomyRefs[k].taxonomyRef);
            }
        }
    }

    return response;
}

function get_category(item) {
    let response = '';

    let k = 0;
    let cats = get_category_array(item);
    for (k = 0; k < cats.length; k++) {
        if (response.length > 0) {
            response += ', ';
        }

        response += cats[k];
    }

    return response;
}

function get_cast(item) {
    return get_talents(item, 'cast');
}

function get_director(item) {
    return get_talents(item, 'director');
}

function get_time(time, type) {

    if (!time) {
        return (type == 'unformatted') ? "00:00:00" : "0 m";
    }

    var result = "";
    const regex = /^\d*$/;

    //356400 is upper bound which is 99 hours
    if (regex.test(time) && time < 356400) {
        let hour_length = Math.floor(time / 3600);
        let min_lenth = Math.floor((time % 3600) / 60);

        if (type == 'unformatted') {
            result = (hour_length < 10 ? '0' : '') + hour_length + ':' + (min_lenth < 10 ? '0' : '') + min_lenth + ':00';
        } else {
            result = (hour_length > 0 ? hour_length + ' h ' : '') + min_lenth + ' m';
        }

    } else {
        result = (type == 'unformatted') ? "00:00:00" : "0 m";
    }

    return result;
}


function get_image(item) {
    let response = '';
    if (item && item.hasOwnProperty('boxArtImage') && item.boxArtImage && item.boxArtImage.length > 0) {
        response = item.boxArtImage;
    } else if (item && item.hasOwnProperty('stillImage') && item.stillImage && item.stillImage.length > 0) {
        response = item.stillImage;
    } else if (item && item.hasOwnProperty('marquee') && item.marquee && item.marquee.length > 0) {
        response = item.marquee;
    }

    return response;
}

function get_rating(item) {

    if (!item) {
        return 'Not Available';
    }

    return (
        (!(typeof item.fccRating === 'undefined' || item.fccRating === null)) ?
        (item.fccRating.rating ? item.fccRating.rating : 'Not Available') :
        (!(typeof item.mpaaRating === 'undefined' || item.mpaaRating === null) ?
            (item.mpaaRating.rating ? item.mpaaRating.rating : 'Not Available') : 'Not Available'
        )
    );
}

function get_year(item) {

    if (!item) {
        return '';
    }

    let year = ((item.hasOwnProperty('releaseDate') && item.releaseDate) ? utils.timeFormat(parseInt(item.releaseDate * 1000), 'yyyy') : "");
    if (year.match(/^\d+$/)) {
        let yearNum = parseInt(year);
        return yearNum >= 1970 && yearNum <= new Date().getFullYear() ? year : "";
    } else {
        return "";
    }
}

function get_trailer_id(item) {
    let response = '';
    if (item && item.hasOwnProperty('trailer') && item.trailer) {
        response = item.trailer;
    }

    return response;
}

function load_episode(seasonTitle, epis) {
    try {
        let new_sub = new SubMediaItem();

        if (epis && epis.hasOwnProperty('id') && epis.id) {
            new_sub.ID = epis.id;
        } else {
            return new_sub;
        }
        new_sub.Title = epis && epis.title || "";
        new_sub.UropID = urop(seasonTitle, epis && epis.seasonNum, epis && epis.episodeNum);
        new_sub.ShortDescription = "";
        new_sub.ImageURI = get_image(epis);
        new_sub.Description = epis && epis.synopsis || "";
        new_sub.DurationUnformated = get_time(epis.duration, 'unformatted');
        new_sub.DurationFormatted = get_time(epis.duration, 'formatted');
        new_sub.PurchasePrice = 0;
        new_sub.Review = 'Not available';
        new_sub.Copyright = 'Not available';
        new_sub.CountryOrigin = 'Not available';
        new_sub.Category = get_category(epis);
        let cast = get_cast(epis);
        new_sub.Cast = ((cast && cast.length > 0) ? cast : 'Not available');
        let direct = get_director(epis);
        new_sub.Director = ((direct && direct.length > 0) ? direct : 'Not available');
        new_sub.IsDRM = ((epis && epis.encrypted) ? "true" : "false");
        new_sub.Type = 'movie';
        new_sub.Rating = get_rating(epis);
        new_sub.applyFilter(filters.filters.taxonomyRef);
        new_sub.Year = get_year(epis);
        new_sub.SubtitleEnable = '';
        new_sub.AudioEnable = '';
        new_sub.CCEnable = '';

        return new_sub;

    } catch (Error) {
        console.log(Error.message);
    }
}

function load_metadata(item) {
    if (!(item && item.id)) {
        return;
    }

    try {
        obj = [];
        obj = new MediaItem();
        obj.ID = item.id;

        obj.Type = (
            (item.type && (item.type == 'TV' || item.type == 'TVSEASON' || item.type == 'TVSERIES')) ?
            'tv' :
            (!item.type ? '' : 'movie')
        );

        if (item.type && item.type == 'TVSEASON') {
        	seriesId = item.seriesId || ""
            let curtvseries = jsonQuery('tvshows[id=${seriesId}]', { data: tvshows }).value || {
                synopsis: '',
                Rating: '',
                drm: (item.encrypted ? "true" : "false"),
                title: ''
            };

            if (curtvseries) {
                obj.ShortDescription = "";
                obj.Description = curtvseries.synopsis;
                obj.Rating = curtvseries.Rating;
                obj.IsDRM = curtvseries.drm;
                obj.ImageURI = curtvseries && curtvseries.boxArtImage || "";
                obj.Year = get_year(curtvseries);
                obj.Title = curtvseries.title;
            } else {
                obj.ShortDescription = "";
                obj.Description = (item.type == 'TVSEASON' ? '' : (!item.synopsis ? '' : item.synopsis));
                obj.Rating = ((item.type == 'TV') ? (item.fccRating && item.fccRating.rating || "") : (item.type == 'TVSEASON' ? 'Not Available' : (item.mpaaRating && item.mpaaRating.rating || "")));
                obj.IsDRM = (item.encrypted ? "true" : "false");
                obj.ImageURI = '';
                obj.Year = (item.hasOwnProperty('releaseDate') ? utils.timeFormat(parseInt(item.releaseDate), 'yyyy') : utils.timeFormat(null, 'yyyy-MM-dd'));
                obj.Title = item.title || "";
            }
        } else {
            obj.Title = item.title || "";
            obj.ShortDescription = "";
            obj.Description = item.synopsis || "";
            obj.Rating = get_rating(item);
            obj.ImageURI = get_image(item);
            obj.IsDRM = (item.encrypted ? "true" : "false");
            obj.Year = get_year(item);

        }

        obj.UropID = urop(obj.Title, item.seasonNum, item.episodeNum);
        obj.Year = get_year(item);
        obj.DurationUnformated = (item.type == 'TVSEASON' ? '' : get_time(item.duration, 'unformatted'));
        obj.DurationFormatted = (item.type == 'TVSEASON' ? '' : get_time(item.duration, 'formatted'));
        obj.Review = 'Not available';
        obj.Copyright = 'Not available';
        obj.CountryOrigin = 'Not available';
        obj.CriticScore = '0.00';
        obj.PeopleScore = '0.00';
        obj.ShortTitle = 'Not available';
        obj.TrailerMediaID = '';
        obj.PurchasePrice = 0;
        obj.Image404URI = '';
        obj.SynopsisURI = '';
        let cast = get_cast(item);
        obj.Cast = ((cast && cast.length > 0) ? cast : 'Not available');
        let direct = get_director(item);
        obj.Director = ((direct && direct.length > 0) ? direct : 'Not available');
        obj.Category = get_category(item);

        obj.applyFilter(filters.filters.taxonomyRef);

        obj.AudioEnable = '';
        obj.SubtitleEnable = '';
        obj.CCEnable = '';

        return obj;
    } catch (Error) {
        console.log(Error.message);
    }
}

function push_if_not_exist(mediaItems, item) {

    if (!(item && item.ID)) {
        return mediaItems;
    }

    if (!(mediaItems.find(x => x.ID == item.ID))) {
        mediaItems.push(item);
    }
    return mediaItems;
}

function errorHandler(finalStr, error) {
    console.log("errorHandler()");
    let defaultSchema;

    if (error) {
        defaultSchema = JSON.parse(transform("[]"));
        defaultSchema["ErrorMessage"] = error.errno;
        defaultSchema["Message"] = "Failed";
        return JSON.stringify(defaultSchema);
    } else {
        return finalStr;
    }
}

function transform(text) {
    console.log("transform()");
    try {

        if (typeof text == "undefined" || text == "undefined" || text == "") {
            console.log("transform -> typeof text: undefined or empty")
            text = "[]";
        }

        let items = JSON.parse(text);
        console.log("transform -> items");
        items = !items ? [] : items;

        let mediaItems = {
            'mItems': items
        }
        console.log("transform -> mediaItems");

        let results = jsonQuery('mItems[type=STATIC & title=TV\ Shows]', {
            data: mediaItems
        }).value || {
            collectionRefList: []
        }

        tvshows.tvshows = results.collectionRefList;
        console.log("transform -> tvshows.tvshows");
        for (let ind = 0; ind < tvshows.tvshows.length; ind++) {
            let tvseries = tvshows.tvshows[ind];

            let seriesId = tvseries.id;

            let seriesDetail = jsonQuery('mItems[type=TVSEASON & seriesId=${seriesId}]', {
                data: mediaItems
            }).value || {
                mediaRefList: [{}]
            };

            let firstTV;
            if ((seriesDetail.mediaRefList || []).length > 0) {
                firstTV = seriesDetail.mediaRefList[0];
            } else {
                firstTV = {};
            }

            let rating = (
                (!(typeof firstTV.fccRating === 'undefined' || firstTV.fccRating === null)) ?
                (firstTV.fccRating.rating ? firstTV.fccRating.rating : 'Not Available') :
                (!(typeof firstTV.mpaaRating === 'undefined' || firstTV.mpaaRating === null) ?
                    (firstTV.mpaaRating.rating ? firstTV.mpaaRating.rating : 'Not Available') : 'Not Available'
                )
            );

            let imageUrl = get_image(firstTV);

            let drm = (firstTV.encrypted ? "true" : "false");

            tvseries["Rating"] = rating;
            tvseries["stillImage"] = imageUrl;
            tvseries["drm"] = drm;
            tvseries["releaseDate"] = firstTV.releaseDate || '';
        }

        console.log("transform -> catalog");
        let catalog = {};
        catalog['InnerObject'] = [];
        catalog['Message'] = "Success";
        catalog['Success'] = true;
        catalog['ErrorMessage'] = "";
        catalog['ActionTimeUTC'] = new Date().toISOString().replace(/T/, ' ').replace(/\..+/, '');

        let objM = MediaSuperCategory.New();
        objM.ID = "100";
        objM.Title = "Movies";
        objM.MediaSubCategories = [];
        objT = MediaSuperCategory.New();
        objT.ID = "200";
        objT.Title = "TV";
        objT.MediaSubCategories = [];
        objK = MediaSuperCategory.New();
        objK.ID = "300";
        objK.Title = "Kids";
        objK.MediaSubCategories = [];
        let catalog_movies = [objM, objT, objK];
        let categories = [
            [],
            [],
            []
        ]; //0 - movie, 1 tv, 2 kids

        let AllMoviesubCat = MediaSubCategory.New();
        AllMoviesubCat.ID = '101';
        AllMoviesubCat.Title = 'All';
        catalog_movies[0].MediaSubCategories.push(AllMoviesubCat);
        let AllTVsubCat = MediaSubCategory.New();
        AllTVsubCat.ID = '201';
        AllTVsubCat.Title = 'All';
        catalog_movies[1].MediaSubCategories.push(AllTVsubCat);
        let AllKidsubCat = MediaSubCategory.New();
        AllKidsubCat.ID = '301';
        AllKidsubCat.Title = 'All';
        catalog_movies[2].MediaSubCategories.push(AllKidsubCat);
        categories[0].push('All');
        categories[1].push('All');
        categories[2].push('All');


        let i = 0;
        let tv_collection = [];
        let movie_collection = [];
        let tv_series = [];

        console.log("transform -> foreach items");
        for (i = 0; i < items.length; i++) {
            if (items[i] && items[i].mediaRefList && items[i].mediaRefList.length > 0) {
                if (items[i].type == 'TVSEASON') {

                    let curr_genres = [];
                    let j = 0;
                    let resp = load_metadata(items[i]);

                    if (resp.Title == '')
                        resp.Title = items[i].title;

                    for (j = 0; j < items[i].mediaRefList.length; j++) {
                        resp.SubMediaItems.push(load_episode(resp.Title, items[i].mediaRefList[j]));
                    }

                    if (items[i].mediaRefList.length > 0) {
                        curr_genres = load_episode(resp.Title, items[i].mediaRefList[0]).Category.split(',');
                        resp.ImageURI = get_image(items[i].mediaRefList[0]);
                    }

                    let k = 0;
                    for (k = 0; k < curr_genres.length; k++) {
                        let loc = 1;
                        let cat_key = categories[loc].indexOf(curr_genres[k]);
                        if (curr_genres[k].toLowerCase().indexOf('kids')) {
                            if (cat_key == -1) {
                                cat_key = categories[loc].length;
                                categories[loc].push(curr_genres[k]);

                                let newSubcat = MediaSubCategory.New();
                                newSubcat.ID = '' + (100 * (loc + 1) + cat_key + 1);
                                newSubcat.Title = curr_genres[k];

                                catalog_movies[loc].MediaSubCategories.push(newSubcat);
                            }

                            push_if_not_exist(catalog_movies[loc].MediaSubCategories[cat_key].MediaItems, resp);
                            push_if_not_exist(catalog_movies[loc].MediaSubCategories[0].MediaItems, resp);

                        }
                        if (curr_genres[k].toLowerCase().indexOf('kids') > -1) {
                            let loc = 2;
                            let cat_key = categories[loc].indexOf('TV');
                            if (cat_key == -1) {
                                cat_key = categories[loc].length;
                                categories[loc].push('TV');

                                let newSubcat = MediaSubCategory.New();
                                newSubcat.ID = '' + (100 * (loc + 1) + cat_key + 1);
                                newSubcat.Title = 'TV';

                                catalog_movies[loc].MediaSubCategories.push(newSubcat);
                            }

                            push_if_not_exist(catalog_movies[loc].MediaSubCategories[cat_key].MediaItems, resp);
                            push_if_not_exist(catalog_movies[loc].MediaSubCategories[0].MediaItems, resp);
                        }
                    }

                } else if (items[i].type == 'STATIC') {
                    let j = 0;

                    for (j = 0; j < items[i].mediaRefList.length; j++) {
                        let curr_genres = [];

                        let resp = load_metadata(items[i].mediaRefList[j]);

                        let cates = resp.Category.split(',');

                        for (let ind = 0; ind < cates.length; ind++) {
                            if (curr_genres.indexOf(cates[ind].trim()) < 0) {
                                curr_genres.push(cates[ind].trim());
                            }
                        }

                        let k = 0;
                        for (k = 0; k < curr_genres.length; k++) {
                            if (resp.Type == 'movie') {
                                if (curr_genres[k].toLowerCase().indexOf('kids')) {
                                    let loc = 0;
                                    let cat_key = categories[loc].indexOf(curr_genres[k]);
                                    if (cat_key == -1) {
                                        cat_key = categories[loc].length;
                                        categories[loc].push(curr_genres[k]);
                                        let newSubcat = MediaSubCategory.New();
                                        newSubcat.ID = '' + (100 * (loc + 1) + cat_key + 1);
                                        newSubcat.Title = curr_genres[k];
                                        catalog_movies[loc].MediaSubCategories.push(newSubcat);
                                    }
                                    push_if_not_exist(catalog_movies[loc].MediaSubCategories[cat_key].MediaItems, resp);
                                    push_if_not_exist(catalog_movies[loc].MediaSubCategories[0].MediaItems, resp);
                                }
                                if (curr_genres[k].toLowerCase().indexOf('kids') > -1) {
                                    let loc = 2;
                                    let cat_key = categories[loc].indexOf('Movies');
                                    if (cat_key == -1) {
                                        cat_key = categories[loc].length;
                                        categories[loc].push('Movies');

                                        let newSubcat = MediaSubCategory.New();
                                        newSubcat.ID = '' + (100 * (loc + 1) + cat_key + 1);
                                        newSubcat.Title = 'Movies';

                                        catalog_movies[loc].MediaSubCategories.push(newSubcat);
                                    }

                                    push_if_not_exist(catalog_movies[loc].MediaSubCategories[cat_key].MediaItems, resp);
                                    push_if_not_exist(catalog_movies[loc].MediaSubCategories[0].MediaItems, resp);
                                }
                            }
                        }

                    }
                }
            }
        }

        let minimal_items = [3, 4];

        for (i = 0; i < catalog_movies.length; i++) {
            let cat;
            for (cat = 0; cat < catalog_movies[i].MediaSubCategories.length; cat++) {
                let j;
                for (j = catalog_movies[i].MediaSubCategories[cat].MediaItems.length; j < minimal_items[i]; j++) {
                    catalog_movies[i].MediaSubCategories[cat].MediaItems.push({});
                }
                catalog_movies[i].MediaSubCategories[cat].MediaItems.sort(
                                    (a, b) => {
                                        if (a.Title && b.Title)
                                        {
                                            let title_a = a.Title.toLowerCase();
                                            let title_b = b.Title.toLowerCase();
                                            return  title_a.localeCompare(title_b);
                                        }
                                    }
                                );

            }
        }

        catalog['InnerObject'] = catalog_movies;

        catalog['InnerObject'][2].MediaSubCategories.shift();
        console.log("transform -> catalog: " + catalog);
        finalStr = JSON.stringify(catalog);
        console.log("transform -> finalStr");
        return finalStr;
    } catch (Error) {
        console.log("transform -> catch error -> " + Error.message);
        // app hangs here, so return something to let the process end
        defaultSchema = JSON.parse(transform("[]"));
        defaultSchema["ErrorMessage"] = Error.errno;
        defaultSchema["Message"] = "Error parsing GOGO collections JSON";
        return JSON.stringify(defaultSchema);
    }
}
exports.errorHandler = errorHandler;
exports.transform = transform;
exports.get_talents = get_talents;
exports.get_category_array = get_category_array;
exports.get_category = get_category;
exports.get_time = get_time;
exports.get_image = get_image;
exports.get_rating = get_rating;
exports.get_year = get_year;
exports.get_trailer_id = get_trailer_id;
exports.load_episode = load_episode;
exports.load_metadata = load_metadata;
exports.push_if_not_exist = push_if_not_exist;
